package edu.blinded.authors.tdrExperimentAnalysis;

import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.function.Function;

public class AnalyzeExperimentLogs {

    public static void main(String[] args) throws Exception {
        try {
            analyzeLogs(getFiles());
        } catch (final Exception e) {
            System.out.println();
            e.printStackTrace(System.out);
        }
    }

    public static String[] getFiles() {
        return new String[] {
                "results/experiment.1518074505769.log",
                "results/experiment.1518527667727.log",
                "results/experiment.1518692804822.log",
                "results/experiment.1518696216817.log",
                "results/experiment.1518779076232.log",
                "results/inferredToggles.log"
        };
    }

    private static void analyzeLogs(String... filenames) throws IOException {
        final LogRecords log = LogRecords.load(filenames);
        final Collection<SessionData> sessions = log.getSplitByParticipant();
        final int NO_TIME_LIMIT = 99999;

        System.out.println("Number of sessions: " + sessions.size());

        final Map<String, Integer> countsPerState = getCountsPerLastState(sessions);
        for (final Entry<String, Integer> e : countsPerState.entrySet()) {
            System.out.println("  last state " + e.getKey() + ": " + e.getValue());
        }

        for (final SessionData s : sessions) {
            final Collection<String> unclassified = s.getUnclassifiedRemarks();
            if (!unclassified.isEmpty()) {
                System.out.println("== unclassified remarks " + s.getId());
                for (final String r : unclassified) {
                    System.out.println(r.replace("\"", "\\\"").replace(": #", ": putDef(ret, \"#") + "\", );");
                }
            }
        }

        sessions.removeIf((SessionData s) -> !(s.isRealEffort(0) || s.isRealEffort(1)));
        System.out.println("Number of sessions with at least one review completed: " + sessions.size());
        System.out.println("Number of sessions with all needed data completed: " + sessions.stream().filter(AnalyzeExperimentLogs::neededDataComplete).count());

        System.out.println();


        final Map<String, Function<SessionData, ? extends Object>> columns = new LinkedHashMap<>();

        columns.put("pId", (SessionData s) -> "p" + s.getId());
        columns.put("shortId", SessionData::getShortId);
        columns.put("totalDuration", SessionData::getTotalDuration);
        columns.put("excludeAsDuplicate", SessionData::isDuplicateParticipation);
        columns.put("role", (SessionData s) -> s.getVariable("role"));
        columns.put("profDevExp", (SessionData s) -> s.getVariable("profDevExp"));
        columns.put("javaExp", (SessionData s) -> s.getVariable("javaExp"));
        columns.put("programPract", (SessionData s) -> s.getVariable("programPract"));
        columns.put("reviewPract", (SessionData s) -> s.getVariable("reviewPract"));
        columns.put("workStartHours", (SessionData s) -> toNumber(s.getVariable("workStartHours")));
        columns.put("fitness", (SessionData s) -> s.getVariable("fitness"));
        columns.put("treatmentCombination", SessionData::getTreatmentCombination);
        columns.put("viewWidth", SessionData::getViewWidth);
        columns.put("viewHeight", SessionData::getViewHeight);
        columns.put("screenWidth", SessionData::getScreenWidth);
        columns.put("screenHeight", SessionData::getScreenHeight);
        columns.put("browser", SessionData::getBrowser);
        columns.put("treatmentR0", (SessionData s) -> s.getTreatment(0));
        columns.put("changeR0", (SessionData s) -> s.getChange(0));
        columns.put("netTimeForReviewWithoutIdlingR0", (SessionData s) -> s.isRealEffort(0) ? s.getNetTimeForReview(0, false) : "NA");
        columns.put("netTimeForReviewWithIdlingR0", (SessionData s) -> s.isRealEffort(0) ? s.getNetTimeForReview(0, true) : "NA");
        columns.put("netTimeForReviewWithIdlingProdOnlyR0", (SessionData s) -> s.isRealEffort(0) ? s.getProdTimeForReview(0) : "NA");
        columns.put("grossTimeForReviewR0", (SessionData s) -> s.getGrossTimeForReview(0));
        columns.put("commentedBothFilesR0", (SessionData s) -> s.isRealEffort(0) ? s.commentedBothFiles(0) : "NA");

        // PROD MAINTAINABILITY
        columns.put("prodMaintIssuesFoundR0", (SessionData s) -> isComplete(s, 0) ? s.getProdMaintainabilityIssues(0) : "NA");
        columns.put("prodMaintIssuesTotalR0", (SessionData s) -> isComplete(s, 0) ? s.getTotalNumberOfIssues(0,"ProdMaint") : 0);

        // TEST MAINTAINABILITY
        columns.put("testMaintIssuesFoundR0", (SessionData s) -> isComplete(s, 0) ? s.getTestMaintainabilityIssues(0) : "NA");
        columns.put("testMaintIssuesTotalR0", (SessionData s) -> isComplete(s, 0) ? s.getTotalNumberOfIssues(0,"TestMaint") : "NA");

        // PROD BUGS
        columns.put("prodBugsFoundR0", (SessionData s) -> isComplete(s, 0) ? s.getProdBugs(0) : "NA");
        columns.put("prodBugsTotalR0", (SessionData s) -> isComplete(s, 0) ? s.getTotalNumberOfIssues(0,"ProdBugs") : "NA");

        // TEST BUGS
        columns.put("testBugsFoundR0", (SessionData s) -> isComplete(s, 0) ? s.getTestBugs(0) : "NA");
        columns.put("testBugsTotalR0", (SessionData s) -> isComplete(s, 0) ? s.getTotalNumberOfIssues(0,"TestBugs") : "NA");

        columns.put("timeForUnderstandingR0", (SessionData s) -> s.getGrossTimeForUnderstanding(0));
        columns.put("understandingCorrectR0", (SessionData s) -> s.getUnderstandingCorrect(0));
        columns.put("understandingTotalR0", (SessionData s) -> s.getUnderstandingTotal(0));
        columns.put("scrollAmountR0", (SessionData s) -> s.getScrollAmount(0));
        columns.put("pauseCountR0", (SessionData s) -> s.getPauseCount(0));
        columns.put("descriptionCountR0", (SessionData s) -> s.getDescriptionCount(0));
        columns.put("toggleCountR0", (SessionData s) -> s.getToggleCount(0));
        columns.put("treatmentR1", (SessionData s) -> s.getTreatment(1));
        columns.put("changeR1", (SessionData s) -> s.getChange(1));
        columns.put("netTimeForReviewWithoutIdlingR1", (SessionData s) -> s.isRealEffort(1) ? s.getNetTimeForReview(1, false) : "NA");
        columns.put("netTimeForReviewWithIdlingR1", (SessionData s) -> s.isRealEffort(1) ? s.getNetTimeForReview(1, true) : "NA");
        columns.put("netTimeForReviewWithIdlingProdOnlyR1", (SessionData s) -> s.isRealEffort(0) ? s.getProdTimeForReview(1) : "NA");
        columns.put("grossTimeForReviewR1", (SessionData s) -> s.getGrossTimeForReview(1));
        columns.put("commentedBothFilesR1", (SessionData s) -> s.commentedBothFiles(1));

        // PROD MAINTAINABILITY
        columns.put("prodMaintIssuesFoundR1", (SessionData s) -> isComplete(s, 1) ? s.getProdMaintainabilityIssues(1) : "NA");
        columns.put("prodMaintIssuesTotalR1", (SessionData s) -> isComplete(s, 1) ? s.getTotalNumberOfIssues(1,"ProdMaint") : 0);

        // TEST MAINTAINABILITY
        columns.put("testMaintIssuesFoundR1", (SessionData s) -> isComplete(s, 1) ? s.getTestMaintainabilityIssues(1) : "NA");
        columns.put("testMaintIssuesTotalR1", (SessionData s) -> isComplete(s, 1) ? s.getTotalNumberOfIssues(1,"TestMaint") : "NA");

        // PROD BUGS
        columns.put("prodBugsFoundR1", (SessionData s) -> isComplete(s, 1) ? s.getProdBugs(1) : "NA");
        columns.put("prodBugsTotalR1", (SessionData s) -> isComplete(s, 1) ? s.getTotalNumberOfIssues(1,"ProdBugs") : "NA");

        // TEST BUGS
        columns.put("testBugsFoundR1", (SessionData s) -> isComplete(s, 1) ? s.getTestBugs(1) : "NA");
        columns.put("testBugsTotalR1", (SessionData s) -> isComplete(s, 1) ? s.getTotalNumberOfIssues(1,"TestBugs") : "NA");

        columns.put("timeForUnderstandingR1", (SessionData s) -> s.getGrossTimeForUnderstanding(1));
        columns.put("understandingCorrectR1", (SessionData s) -> s.getUnderstandingCorrect(1));
        columns.put("understandingTotalR1", (SessionData s) -> s.getUnderstandingTotal(1));
        columns.put("scrollAmountR1", (SessionData s) -> s.getScrollAmount(1));
        columns.put("pauseCountR1", (SessionData s) -> s.getPauseCount(1));
        columns.put("descriptionCountR1", (SessionData s) -> s.getDescriptionCount(1));
        columns.put("toggleCountR1", (SessionData s) -> s.getToggleCount(1));
        columns.put("comparisonUnderstanding", SessionData::interpretComparisonUnderstanding);
        columns.put("comparisonComplicated", SessionData::interpretComparisonComplicated);
        columns.put("pageReloads", SessionData::determinePageReloads);
        columns.put("q_C1_A", SessionData::questionCorrectC1A);
        columns.put("q_C1_B", SessionData::questionCorrectC1B);
        columns.put("q_C1_C", SessionData::questionCorrectC1C);
        columns.put("q_C2_A", SessionData::questionCorrectC2A);
        columns.put("q_C2_B", SessionData::questionCorrectC2B);
        columns.put("q_C2_C", SessionData::questionCorrectC2C);
        for (final SessionData.Defect d : P1.values()) {
            columns.put("d_D1_" + d, (SessionData s) -> s.foundDefect(s.getReviewNumberWith(PatchToDiff.P1), d));
        }
        for (final SessionData.Defect d : P2.values()) {
            columns.put("d_D2_" + d, (SessionData s) -> s.foundDefect(s.getReviewNumberWith(PatchToDiff.P2), d));
        }

        FileWriter fw = null;
        try {
            fw = new FileWriter("resultAnalysis.csv");

            //print header
            for (final String s : columns.keySet()) {
                fw.append(s + ";");
            }
            fw.append("\n");

            //print rows
            for (final SessionData s : sessions) {
                for (final Function<SessionData, ? extends Object> f : columns.values()) {
                    final Object o = f.apply(s);
                    fw.append(o == null ? "NA" : o.toString() + ";");
                }
                fw.append("\n");
            }

        } catch (final Exception e) {
            System.out.println("Error in CsvFileWriter !!!");
            e.printStackTrace();
        } finally {
            try {
                fw.flush();
                fw.close();
            } catch (final IOException e) {
                System.out.println("Error while flushing/closing fileWriter !!!");
                e.printStackTrace();
            }

        }
        printResults(sessions, columns);

    }

    private static void printResults(Collection<SessionData> sessions, Map<String, Function<SessionData, ?>> columns) {
        //print header
        for (final String s : columns.keySet()) {
            System.out.print(s);
            System.out.print(";");
        }
        System.out.println();

        //print rows
        for (final SessionData s : sessions) {
            for (final Function<SessionData, ? extends Object> f : columns.values()) {
                final Object o = f.apply(s);
                System.out.print(o == null ? "NA" : o.toString());
                System.out.print(";");
            }
            System.out.println();
        }
    }

    private static Map<String, Integer> getCountsPerLastState(Collection<SessionData> sessions) {
        final Map<String, Integer> ret = new LinkedHashMap<>();
        for (final SessionData s : sessions) {
            final String lastState = s.getLastState();
            final Integer oldCount = ret.get(lastState);
            ret.put(lastState, oldCount == null ? 1 : oldCount + 1);
        }
        return ret;
    }

    static double toNumber(String variable) {
        return Double.parseDouble(variable.replace(',', '.'));
    }

    private static boolean neededDataComplete(SessionData s) {
        return s.isRealEffort(0) && s.isRealEffort(1);
    }

    private static boolean isComplete(SessionData s, int reviewNumber){
//        return s.isRealEffort(reviewNumber) && s.commentedBothFiles(reviewNumber);
        return s.isRealEffort(reviewNumber);
    }

}
