package edu.blinded.authors.tdrExperimentAnalysis;

import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import edu.blinded.authors.tdrExperimentAnalysis.SessionData.RemarkClassifier;

public class CrossCheckToggles {

    public static void main(String[] args) throws IOException {
        final LogRecords log = LogRecords.load(AnalyzeExperimentLogs.getFiles());
        final Collection<SessionData> sessions = log.getSplitByParticipant();

        final Set<String> okSessions = new TreeSet<>();
        final Set<String> notOkSessions = new TreeSet<>();
        for (final SessionData session : sessions) {
            if (!session.isRealEffort(1)) {
                continue;
            }
            boolean ok;
            ok = crossCheckToggles(session, 0);
            ok |= crossCheckToggles(session, 1);
            if (ok) {
                okSessions.add(session.getId());
            } else {
                notOkSessions.add(session.getId());
            }
        }
        System.out.println("ok: " + okSessions.size());
        System.out.println("not ok: " + notOkSessions.size());
    }

    private static boolean crossCheckToggles(SessionData session, int i) {
        final List<LogRecord> reviewRecords = session.getReviewRecords(i);
        if (!reviewRecords.isEmpty()) {
            return crossCheckToggles(reviewRecords, session.getTreatment(i), session.createRemarkClassifier(i));
        } else {
            return true;
        }
    }

    private static boolean crossCheckToggles(List<LogRecord> reviewRecords, Treatment t, RemarkClassifier remarkClassifier) {
        if (t == Treatment.OP) {
            return true;
        }
        boolean ret = true;
        boolean inTest = t == Treatment.TF;
        for (final LogRecord r : reviewRecords) {
            final String rest = r.getSuffixOfRest(3).trim();
            if (rest.endsWith("pressedToggleShownCode")) {
                inTest = !inTest;
            }
            if (rest.startsWith("startEnterReviewRemark;")) {
                final String realFn = remarkClassifier.toRealFilename(rest);
                if (isTestFile(realFn) != inTest) {
                    System.out.println("invalid state " + inTest + " for " + r.getSessionId() + ";" + r.getTimestamp() + ";" + r.getRest());
                    ret = false;
                }
            }
        }
        return ret;
    }

    private static boolean isTestFile(String remarkWithRealFn) {
        return remarkWithRealFn.contains(";#bR;")
            || remarkWithRealFn.contains(";#bL;");
    }

}
