package edu.blinded.authors.tdrExperimentAnalysis;

import java.io.IOException;
import java.util.Collection;
import java.util.List;

public class InferToggles {

    private static final long START_ID_WITH_LOG = 257339076232L;

    public static void main(String[] args) throws IOException {
        final LogRecords log = LogRecords.load(AnalyzeExperimentLogs.getFiles());
        final Collection<SessionData> sessions = log.getSplitByParticipant();

        for (final SessionData session : sessions) {
//            if (session.getId().equals("256634505773")) {
//                System.out.println("asdfasdrf");
//            }
            if (needsInferredToggles(session)) {
                if (session.getTreatment(0) != Treatment.OP) {
                    inferToggles(session.getReviewRecords(0), session.getViewHeight(0));
                }
                if (session.getTreatment(1) != Treatment.OP) {
                    inferToggles(session.getReviewRecords(1), session.getViewHeight(1));
                }
            }
        }
    }

    private static boolean needsInferredToggles(SessionData session) {
        return Long.parseLong(session.getId()) < START_ID_WITH_LOG;
    }

    private static void inferToggles(List<LogRecord> reviewRecords, int viewHeight) {
        if (reviewRecords.isEmpty()) {
            return;
        }

        final int endButtonX = determineEndButtonX(reviewRecords);

        for (final LogRecord r : reviewRecords) {
            if (isReviewEnd(r)) {
                break;
            }
            if (isMousedown(r)) {
                final int xDiffToEnd = getClientX(r) - endButtonX;
                final int yDiffToLowerBorder = viewHeight - getClientY(r);
                if (xDiffToEnd <= -78 && yDiffToLowerBorder <= 253) {
                    printToggleRecord(r);
                }
            }
        }

    }

    private static void printToggleRecord(LogRecord r) {
        System.out.println(r.getSessionId() + ";" + r.getTimestamp() + ";" + r.getRest().replace(r.getSuffixOfRest(3), "pressedToggleShownCode"));
    }

    private static int determineEndButtonX(List<LogRecord> reviewRecords) {
        int lastX = -1;
        for (final LogRecord r : reviewRecords) {
            if (isReviewEnd(r)) {
                if (lastX < 0) {
                    throw new AssertionError("no mouseclick before end");
                }
                return lastX;
            }
            if (isMousedown(r)) {
                lastX = getClientX(r);
            }
        }
        throw new AssertionError("no review end found" + reviewRecords.get(0).getRest());
    }

    private static boolean isReviewEnd(LogRecord r) {
        return r.getRest().trim().endsWith("review ended");
    }

    private static boolean isMousedown(LogRecord r) {
        return r.getRestField(3).equals("mdo");
    }

    private static int getClientX(LogRecord r) {
        return Integer.parseInt(r.getRestField(6), 36);
    }

    private static int getClientY(LogRecord r) {
        return Integer.parseInt(r.getRestField(7), 36);
    }
}
