package edu.blinded.authors.tdrExperimentAnalysis;

public class LogRecord {

    private final String sessionId;
    private final long timestamp;
    private final int index;
    private final String rest;

    private LogRecord(int index, String line) {
        this.index = index;

        final int firstSem = line.indexOf(';');
        this.sessionId = line.substring(0, firstSem);
        final String remain = line.substring(firstSem + 1);
        final int secondSem = remain.indexOf(';');
        this.timestamp = Long.parseLong(remain.substring(0, secondSem));
        this.rest = remain.substring(secondSem + 1);
    }

    public static LogRecord create(int index, String line) {
        return new LogRecord(index, line);
    }

    public String getSessionId() {
        return this.sessionId;
    }

    public String getRest() {
        return this.rest;
    }

    public long timeUntil(LogRecord logRecord) {
        return logRecord.timestamp - this.timestamp;
    }

    public long reviewTimeUntil(LogRecord logRecord) {
        return logRecord.getClientTimeReview() - this.getClientTimeReview();
    }

    public long wmspanTimeUntil(LogRecord logRecord) {
        return logRecord.getClientTimeWmspan() - this.getClientTimeWmspan();
    }

    public long getClientTimeReview() {
        return Long.parseLong(this.getRestField(2), 36);
    }

    public long getClientTimeWmspan() {
        return Long.parseLong(this.getRestField(1), 36);
    }

    public String getRestField(int index) {
        final String s = this.getSuffixOfRest(index);
        final int semi = s.indexOf(';');
        if (semi < 0) {
            return s;
        } else {
            return s.substring(0, semi);
        }
    }

    public String getSuffixOfRest(int semisToSkip) {
        String remain = this.rest;
        for (int i = 0; i < semisToSkip; i++) {
            remain = remain.substring(remain.indexOf(';') + 1);
        }
        return remain;
    }

    public long getTimestamp() {
        return this.timestamp;
    }

    public boolean isFromReview() {
        return this.getRest().startsWith("log from review;");
    }

}
