package edu.blinded.authors.tdrExperimentAnalysis;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class LogRecords {

    private final List<LogRecord> logRecords = new ArrayList<>();

    private LogRecords() {
    }

    public static LogRecords load(String[] filenames) throws IOException {
        final LogRecords ret = new LogRecords();
        for (final String file : filenames) {
            ret.addFrom(file);
        }
        return ret;
    }

    private void addFrom(String file) throws IOException {
        try (FileInputStream is = new FileInputStream(file)) {
            final InputStreamReader ir = new InputStreamReader(is, "UTF-8");
            final BufferedReader br = new BufferedReader(ir);
            String line;
            int index = 0;
            while ((line = br.readLine()) != null) {
                this.logRecords.add(LogRecord.create(index++, line));
            }
        }
    }

    public Collection<SessionData> getSplitByParticipant() {
        final Map<String, List<LogRecord>> split = new LinkedHashMap<>();
        for (final LogRecord lr : this.logRecords) {
            List<LogRecord> list = split.get(lr.getSessionId());
            if (list == null) {
                list = new ArrayList<>();
                split.put(lr.getSessionId(), list);
            }
            list.add(lr);
        }
        return split.values().stream().map(SessionData::new).collect(Collectors.toList());
    }

}
