package edu.blinded.authors.tdrExperimentAnalysis;

import static edu.blinded.authors.tdrExperimentAnalysis.SessionData.Defect.putDef;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public enum P2 implements SessionData.Defect {

    // maintainability issues PROD
    PROD_METHOD_DESCRIPTION,
    VARIABLE_NAMES,

    // maintainability issues test
    TEST_METHOD_DESCRIPTION,
    INIT_SPRITE,

    // BUG PROD
    X_Y_GREATER_THAN_ZERO,
    WRONG_RETURN_TYPE,
    TYPO,

    // BUG TEST
    DIFFERENT_NUMBERS,
    WRONG_ASSERT;

    @Override
    public String getShortString() {
        return this.getClass().getSimpleName() + "_" + this.ordinal();
    }

    @Override
    public boolean isDelocalized() {
        return true;
    }

    static Map<String, List<SessionData.Defect>> getTrueDefects() {
        final Map<String, List<SessionData.Defect>> ret = new LinkedHashMap<>();
        putDef(ret, "#aR;38;Commit description says this added some tests, but I don't see them (toggle shown code doesn't do anything)", SessionData.FP.FP);
        putDef(ret, "#aR;40;so no try arbg? that sentence doesn't make sense", SessionData.FP.FP);
        putDef(ret, "#aL;57;whats wrong with keeping this?", SessionData.FP.FP);
        putDef(ret, "#aR;60;please give more useful names to the variables, like \"x - width - 1\"", P2.VARIABLE_NAMES);
        putDef(ret, "#bR;125;isn't always instance of Sprite?", P2.WRONG_ASSERT);
        putDef(ret, "#bR;96;I suggest to put different numbers, like 10, 11, 12, 13", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#aR;60;typo? shouldn't it be y+height-1?", P2.TYPO);
        putDef(ret, "#aR;64;why not return EmptySprite() as before?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#bR;111;doesn't value of x/y argument to sprite() influence the height/width? If it doesn't, you're essentially testing newImage() here.", SessionData.FP.FP);
        putDef(ret, "#bR;125;isn't this condition true regardless of split() arguments?", P2.WRONG_ASSERT);
        putDef(ret, "#bR;97;see line 111", SessionData.FP.FP);
        putDef(ret, "#aR;70;empty method description", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;83;empty method description", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;74;use better intendation, too long list of parameters", SessionData.FP.FP);
        putDef(ret, "#bR;101;review the test method description", SessionData.FP.FP);
        putDef(ret, "#aR;60;x + height - 1..typo? Should be y", P2.TYPO);
        putDef(ret, "#aR;64;why not Empty sprite?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#bR;110;same here", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;96;the number should all be different, since you are testing a specific attribute (width in this case)", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;125;isn't it always the case?", P2.WRONG_ASSERT);
        putDef(ret, "#aR;59;use better intendation, too long list of parameters", SessionData.FP.FP);
        putDef(ret, "#aR;53;Previous and current implementation seems semantically identical.", SessionData.FP.FP);
        putDef(ret, "#aR;55;Where's the description of the method split?", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;55;No documentation for function", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;55;nodoc", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;56;Return value can be more specific: change to ImageSprite", SessionData.FP.FP);
        putDef(ret, "#aR;56;I preferred the old way, with the exception in the if() and the actual code below", SessionData.FP.FP);
        putDef(ret, "#aR;56;check for negatibe numbers", SessionData.FP.FP);
        putDef(ret, "#aR;56;documentation lacking", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;56;Document this method", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;57;what about x + width or y + height being outside image?", SessionData.FP.FP);
        putDef(ret, "#aR;57;What if x/y are negative, always inside image but makes no sense", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#aR;57;what about bounds checking for width and height? is this meant to test if the rect is ENTIRELY within the image?", SessionData.FP.FP);
        putDef(ret, "#aR;57;The old behaviour isn't the same as !withinImage(x, y)", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;57;At this point, I'd ask for a chat instead. I don't have enough context from the information provided to tell if this is valid. (The toggle shown code isn't working so I can't see any tests.)", SessionData.FP.FP);
        putDef(ret, "#aR;57;not same logic", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;57;Prefer exiting early: if (!withinImage(...)) { return ...; }", SessionData.FP.FP);
        putDef(ret, "#aR;57;Much better check", SessionData.FP.FP);
        putDef(ret, "#aR;57;Keep the negation of withinImage here to return early. Reduces amount of code within if blocks and reduces mental stack space.", SessionData.FP.FP);
        putDef(ret, "#aR;57;Invert this if statement so that it returns new ImageSprite early if !withinImage(x, y) to reduce nesting.", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;57;This is a preexisting error, but we should check that x and y are >= 0.", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#aR;57;Unnecessary parentheses?", SessionData.FP.FP);
        putDef(ret, "#aR;57;It might be more readable to reverse this test, and keep the early return of the previous version.", SessionData.FP.FP);
        putDef(ret, "#aR;57;Good job making this more readable by giving the condition a name.Does this need to be a method call? You could just assign the result of the one-liner in the method to a variable with the same telling name.", SessionData.FP.FP);
        putDef(ret, "#aR;57;Negate condition statement to reduce nesting", SessionData.FP.FP);
        putDef(ret, "#aR;58;This block could remain outside the if statement as before, just use !withinImage(). Unnecessary indentation.", SessionData.FP.FP);
        putDef(ret, "#aR;58;Triangular code; please go back to using a guard clause", SessionData.FP.FP);
        putDef(ret, "#aR;58;Should verify that x, y, width and height are all positive integers.", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#aR;58;width and height are never checked", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#aR;59;`x + height - 1` should be `y + height - 1`", P2.TYPO);
        putDef(ret, "#aR;59;This line is difficult to read. Consider splitting, and consider using temporary variable names for the parameters to make it easier to understand)", SessionData.FP.FP);
        putDef(ret, "#aR;59;Fix alignment in parameters passed to drawImage", SessionData.FP.FP);
        putDef(ret, "#aR;59;should be split over 2 lines", SessionData.FP.FP);
        putDef(ret, "#aR;59;why -1 for all these?", SessionData.FP.FP);
        putDef(ret, "#aR;59;Parameters of this should be split over multiple lines so its clearer", SessionData.FP.FP);
        putDef(ret, "#aR;59;where does the image variable come from?  How is it initialized?  How do we know it is not null before using it?", SessionData.FP.FP);
        putDef(ret, "#aR;59;What happens if x + width is greater than image.width? And similarly for height", SessionData.FP.FP);
        putDef(ret, "#aR;59;Why was the last y  changed to an x?", P2.TYPO);
        putDef(ret, "#aR;59;make this call easier to read, if multi line method call use one line per parameter", SessionData.FP.FP);
        putDef(ret, "#aR;59;perhaps add a comment for this line, it's not fully clear to me what's going on here, why subtract 1 from all the dimensions?", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;59;you don't need so many numbers without context or naming: allocate variables if need be, use named methods, use iterables, etc.", P2.VARIABLE_NAMES);
        putDef(ret, "#aR;59;Put every argument on a new line", SessionData.FP.FP);
        putDef(ret, "#aR;59;What do those magic numbers \"1\" mean?", SessionData.FP.FP);
        putDef(ret, "#aR;59;Use variables as parameters to make clear what they mean. Also, this many parameters are a code smell - consider introducing a struct/class to pass to the method. And reconsider using null as a parameter value.", P2.VARIABLE_NAMES);
        putDef(ret, "#aR;59;Bit copy is probably faster than initializing graphics. Is this libgdx?...", SessionData.FP.FP);
        putDef(ret, "#aR;59;add a comment what is going on here", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;60;Should be y+height-1", P2.TYPO);
        putDef(ret, "#aR;60;Should be y + height -1. (?)", P2.TYPO);
        putDef(ret, "#aR;60;should be y + height", P2.TYPO);
        putDef(ret, "#aR;60;x + hight -> y + hight", P2.TYPO);
        putDef(ret, "#aR;60;should use y instead of x parameter 9", P2.TYPO);
        putDef(ret, "#aR;60;is x + height - 1 should be y + height - 1", P2.TYPO);
        putDef(ret, "#aR;60;hard to understand these parameters... consider extracting to local variables", P2.VARIABLE_NAMES);
        putDef(ret, "#aR;60;Penultimate arg changed from y + height - 1 to x + height - 1. On purpose?", P2.TYPO);
        putDef(ret, "#aR;60;y + height - 1 was changed to x + height - 1. Was this intentional? This doesn't seem right.", P2.TYPO);
        putDef(ret, "#aR;60;not same variables as previous version (x/y + height)", P2.TYPO);
        putDef(ret, "#aR;60;Should be \"y + height - 1\" for the 9th parameter of this function call (4th parameter on this line)", P2.TYPO);
        putDef(ret, "#aR;60;Not a new problem, but I find this indentation style very hard to read. Putting each group of arguments on its own line and indenting those lines by only 4 spaces would produce much more comprehensible code", SessionData.FP.FP);
        putDef(ret, "#aR;60;y + height, not x + height", P2.TYPO);
        putDef(ret, "#aR;60;y + height - 1 instead of x", P2.TYPO);
        putDef(ret, "#aR;60;why is this on a new line?", SessionData.FP.FP);
        putDef(ret, "#aR;60;this method call is chained a lot, not sure if that's needed", SessionData.FP.FP);
        putDef(ret, "#aR;60;x + height - 1 -> y + height - 1", P2.TYPO);
        putDef(ret, "#aR;60;should be y + height; which is why the test in the subsequent code isn't failing, because this would cause an out of bounds error most likely", P2.TYPO);
        putDef(ret, "#aR;60;Should this be y + height - 1?", P2.TYPO);
        putDef(ret, "#aR;60;Has to be y + height - 1", P2.TYPO);
        putDef(ret, "#aR;60;Remove linebreak.", SessionData.FP.FP);
        putDef(ret, "#aR;61;tabs on empty line", SessionData.FP.FP);
        putDef(ret, "#aR;62;Would prefer singular return at end with assignment to variable.", SessionData.FP.FP);
        putDef(ret, "#aR;63;Else clause is unnecessary", SessionData.FP.FP);
        putDef(ret, "#aR;63;with the return inside the if we can remove the else", SessionData.FP.FP);
        putDef(ret, "#aR;63;The else can be removed because the return on the previous line will end the if statement.", SessionData.FP.FP);
        putDef(ret, "#aR;63;the else can be omitted since the if returns", SessionData.FP.FP);
        putDef(ret, "#aR;63;else-branching is unnecessary, because the if-block is left with a return", SessionData.FP.FP);
        putDef(ret, "#aR;64;Before you were returning `EmptySprite` - now you're returning `ImageSprite` with no arguments.  Does this mean that EmptySprite is now obsolete?  Perhaps put that in a separate change if so?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;Previously returned EmptySprite", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;Should be new EmptySprite(). (?)", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;Failing condition in else case should be first. this removes the else case", SessionData.FP.FP);
        putDef(ret, "#aR;64;return type changed from `EmptySprite` to `ImageSprite` - is that itentional?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;this used to be an emptysprite", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;old code uses EmptySprite(), is this ok?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;should be new EmptySprite()", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;Shouldn't you return EmptySprite?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;shouldn't this be emptysprite?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;I think this should be return new EmptySprite()", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;Used to return an EmptySprite() in this case", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;Why not keep the EmptySprite?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;Shouldn't this be new EmptySprite(); like in the old version?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;EmptySprite?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;Did you mean EmptySprite?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;Why was this changed from EmptySprite to ImageSprite?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;May be more approprite to throw an exception here rather than returning an object. This seems to be an error state.", SessionData.FP.FP);
        putDef(ret, "#aR;64;return new EmptySprite();", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;this behavior seems wrong: errors shouldn't be silent unless that's documented as such", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;In the original this branch would return a new EmptySprite(); why the change?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;ImageSprite -> EmptySprite?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;No longer returning new EmptySprite() but rather a new ImageSprite(), might behave differently.", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;It used to be EmptySprite()", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;Why not ``return EmptySprite()`` as it used to be previously?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;64;Fail fast. Guards > long if thens. if !within then return empty", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;68;should be named isWithinImage", SessionData.FP.FP);
        putDef(ret, "#aR;68;Maybe premature optimization for now, but a good change nontheless.", SessionData.FP.FP);
        putDef(ret, "#aR;68;no description again", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;68;can these ever be negative?", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#aR;68;No documentation for function", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;68;Questionable use of a function, but it's not a bad choice", SessionData.FP.FP);
        putDef(ret, "#aR;68;not the same logic in refactor", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;68;why create a method when the code is only used in one place? seems useless...", SessionData.FP.FP);
        putDef(ret, "#aR;68;I always prefer to break longer caluclations out into a function like this", SessionData.FP.FP);
        putDef(ret, "#aR;68;Move this to above the preceding function.", SessionData.FP.FP);
        putDef(ret, "#aR;68;Good use of breakout function", SessionData.FP.FP);
        putDef(ret, "#aR;68;Should check if x or y is negative", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#aR;68;isWithinImage seems like a better name to me", SessionData.FP.FP);
        putDef(ret, "#aR;68;Good breakout.", SessionData.FP.FP);
        putDef(ret, "#aR;68;Good making an extra function of this, likely a function to be used often.", SessionData.FP.FP);
        putDef(ret, "#aR;69;Are coordonates between 0 and size -1 ? If not, check boundaries values (width == x and height == y).", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#aR;69;This check is different than the old one. Are you sure it's correct? Please write a test for this", SessionData.FP.FP);
        putDef(ret, "#aR;69;Uhm, didn't the test semantic change now, in the case of `x == image.getWidth(null)`?  It looks correct now to me, but please check.", SessionData.FP.FP);
        putDef(ret, "#aR;69;other way around from previous check", SessionData.FP.FP);
        putDef(ret, "#aR;69;what about negative x, y?", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#aR;69;getWidth/Height null? what? also the change from and to or might be wrong", SessionData.FP.FP);
        putDef(ret, "#aR;69;Can you explain the logic here? There's a lot of change here and I can't prove to myself that the behavior is the same at first glance.", SessionData.FP.FP);
        putDef(ret, "#aR;69;This does not check for the lower bounds of the image.  It only checks if a point is below the images maximum width and/or height", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#aR;69;Should also constrain x,y>=0", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#aR;69;<=", SessionData.FP.FP);
        putDef(ret, "#aR;69;You're inverting the sense of the original condition, so these comparisons should be lax inequalities: <= not <", SessionData.FP.FP);
        putDef(ret, "#aR;69;This handles x == image.getWidth(null) and y == image.getHeight(null) differently from the previous. Unless this was intentional, this is probably a bug.", SessionData.FP.FP);
        putDef(ret, "#aR;69;To not be a breaking change this should technically be <=", SessionData.FP.FP);
        putDef(ret, "#aR;69;check and vs or", SessionData.FP.FP);
        putDef(ret, "#aR;69;Why are we passing in null here to getWidth and getHeight?", SessionData.FP.FP);
        putDef(ret, "#aR;69;why is null passed here, that doesn't make sense", SessionData.FP.FP);
        putDef(ret, "#aR;69;By flipping the comparision, you moved where equalness would branch to", SessionData.FP.FP);
        putDef(ret, "#aR;69;SHould this be <= ? That would match the negation of previous code on line 57.", SessionData.FP.FP);
        putDef(ret, "#aR;69;Original code uses || operator not &&", SessionData.FP.FP);
        putDef(ret, "#aR;69;<= instead of < ?", SessionData.FP.FP);
        putDef(ret, "#aR;69;I don't know what getWidth and getHeight return when null is passed, so I cannot judge whether this is valid or not. However, it is never checked whether x and y are positive or at least 0", SessionData.FP.FP);
        putDef(ret, "#aR;69;This changes the logic of the condition. Using deMorgan is fine, but note that !(c<d) == (c >= d). So: (A < x) OR (B < y) <==> NOT((A >= x) AND (B >= y)) <==> NOT((x <= A) AND (y <= B)). The negation gets rid of the outer NOT, but you still need the smaller-equals, unless there was a bug in the earlier code that you're fixing here.", SessionData.FP.FP);
        putDef(ret, "#aR;69;This changes the logic of the condition. Using deMorgan is fine, but note that !(c<d) == (c >= d). So: (A < x) OR (B < y) <==> NOT((A >= x) AND (B >= y)) <==> NOT((x <= A) AND (y <= B))", SessionData.FP.FP);
        putDef(ret, "#aR;69;1. Why is null being passed in t othese methods? 2. zero checks?", SessionData.FP.FP);
        putDef(ret, "#aR;70;Will not review, no specs.", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;72;This change still doesn't verify negative values for x & y", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#aR;75;x is used for both width and height, this seems incorrect", P2.TYPO);
        putDef(ret, "#aR;79;should return new EmptySprite()", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;84;Do we need to check > 0 too?", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#bR;86;Can refactor all the code, loads of common operation", P2.INIT_SPRITE);
        putDef(ret, "#bR;86;no need for this sort of documentation for test code", SessionData.FP.FP);
        putDef(ret, "#bR;86;I don't think we need JavaDoc for test code", SessionData.FP.FP);
        putDef(ret, "#bR;87;an -> a", SessionData.FP.FP);
        putDef(ret, "#bR;87;Typo \"an\"", SessionData.FP.FP);
        putDef(ret, "#bR;87;\"an split\" should be \"a split\"", SessionData.FP.FP);
        putDef(ret, "#bR;87;Typo: should be \"a split sprite\"", SessionData.FP.FP);
        putDef(ret, "#bR;87;Typo? And the test name is not descriptive", SessionData.FP.FP);
        putDef(ret, "#bR;87;a split sprite", SessionData.FP.FP);
        putDef(ret, "#bR;87;this also seems sorta off: is this being used to verify things during runtime? that's bad if it is", SessionData.FP.FP);
        putDef(ret, "#bR;87; a split", SessionData.FP.FP);
        putDef(ret, "#bR;89;Note necessary for tests", SessionData.FP.FP);
        putDef(ret, "#bR;90;Formatting problem", SessionData.FP.FP);
        putDef(ret, "#bR;93;method name does not describe the tested feature", SessionData.FP.FP);
        putDef(ret, "#bR;93;name is misleading", SessionData.FP.FP);
        putDef(ret, "#bR;93;Unit tests should not load resources from the file system.", SessionData.FP.FP);
        putDef(ret, "#bR;93;These three tests are nearly identical. Extract common code to simplify them.", P2.INIT_SPRITE);
        putDef(ret, "#bR;94;Inicialization for sprite move to other method. It's always the same.", P2.INIT_SPRITE);
        putDef(ret, "#bR;94;it looks like multiple tests are creating a sprite store and then loading a sprite from it, perhaps the store should be a member variable of the test class? the test now depends on creating an instance of the store", P2.INIT_SPRITE);
        putDef(ret, "#bR;94;You could probably factor out the test sprite loading code, because it is so boring.", P2.INIT_SPRITE);
        putDef(ret, "#bR;95;Magic String", SessionData.FP.FP);
        putDef(ret, "#bR;95;Hardcoded Path", SessionData.FP.FP);
        putDef(ret, "#bR;95;Tests should not rely on disk files or db.", SessionData.FP.FP);
        putDef(ret, "#bR;95;Depends on context but the path shouldn't be hardcoded here", SessionData.FP.FP);
        putDef(ret, "#bR;95;the test now fails when the sprite could not be loaded, the test result should be inconclusive since the method to test was not run at all", SessionData.FP.FP);
        putDef(ret, "#bR;95;this seems rather fragile: any other way of loading images?", SessionData.FP.FP);
        putDef(ret, "#bR;95;Move this to a setup method, read from fs = :(", P2.INIT_SPRITE);
        putDef(ret, "#bR;95;can be done in a @Before method", P2.INIT_SPRITE);
        putDef(ret, "#bR;96;Width/height confusion won't be caught by this test - make it 10, 11, 13, 14?", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;96;Magic Numbers", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;96;why is the method called splitWidth when sprite.split seems to split both width and height? explain", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;96;hardcoded numbers", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;96;Magic numbers", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;96;It seems as though this chain of events could be simplified, some method other than storing 3 objects to get a split", SessionData.FP.FP);
        putDef(ret, "#bR;97;Consider using parameters instead of magic numbers", SessionData.FP.FP);
        putDef(ret, "#bR;97;Magic Number",P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;97;assert width", SessionData.FP.FP);
        putDef(ret, "#bR;97;Above, you're passing null to getWidth(); here, you aren't. At least, your style should be consistent, even if both work", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;97;Why 13?", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;97;This is missing some assertions. Most obviously, if it's going to assert the width, it should also assert the height. Another set of assertions is to assert that the pixels from the split sprite are, pixel-by-pixel, the same as those from the original sprite.", SessionData.FP.FP);
        putDef(ret, "#bR;97;When the sprite is smaller than given size, 13 migt not be the size of the resulting sprite (it can be clipped if original size < 13)", SessionData.FP.FP);
        putDef(ret, "#bR;101;an -> a", SessionData.FP.FP);
        putDef(ret, "#bR;101;Typo \"an\"", SessionData.FP.FP);
        putDef(ret, "#bR;101;again, \"a split sprite\"", SessionData.FP.FP);
        putDef(ret, "#bR;101;Test name is not descriptive", P2.TEST_METHOD_DESCRIPTION);
        putDef(ret, "#bR;101;a split sprite, and the description for this test is the same as the one above, more reason that both should be merged to a single test", P2.TEST_METHOD_DESCRIPTION);
        putDef(ret, "#bR;101;The documentation doesn't make clear how this is different from splitWidth.", P2.TEST_METHOD_DESCRIPTION);
        putDef(ret, "#bR;103;not for tests", SessionData.FP.FP);
        putDef(ret, "#bR;104;Formatting problem", SessionData.FP.FP);
        putDef(ret, "#bR;106;this doesn't have a good name for a test", SessionData.FP.FP);
        putDef(ret, "#bR;106;I would also expect some test cases (and code) with width/height out of bounds", SessionData.FP.FP);
        putDef(ret, "#bR;107;Why are splitWidth and splitHeight seperate tests? TYhey both split in width and height...", SessionData.FP.FP);
        putDef(ret, "#bR;107;this name doesn't match the documentation and tests", SessionData.FP.FP);
        putDef(ret, "#bR;107;This is essentially the same test as splitWidth above; the two can be merged.", SessionData.FP.FP);
        putDef(ret, "#bR;108;Combine this and width into one test maybe?", SessionData.FP.FP);
        putDef(ret, "#bR;109;Magic string", SessionData.FP.FP);
        putDef(ret, "#bR;109;hardCoded path", SessionData.FP.FP);
        putDef(ret, "#bR;109;Tests should not rely on disk files or db.", SessionData.FP.FP);
        putDef(ret, "#bR;109;same as above: this could break pretty easily", SessionData.FP.FP);
        putDef(ret, "#bR;110;And again - Width/height confusion won't be caught by this test - make it 10, 11, 13, 14?", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;110;Magic Numbers", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;110;Code duplicated with splitWidth and splitOutOfBounds", SessionData.FP.FP);
        putDef(ret, "#bR;110;hardcoded numbers", SessionData.FP.FP);
        putDef(ret, "#bR;110;Same as above", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;110;the same method of the sprint is called, even with the same arguments as the above test, shouldn't that be a single test, with multiple asserts?", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;111;Consider using parameters instead of magic numbers", SessionData.FP.FP);
        putDef(ret, "#bR;111;We should use different numbers for the width and height", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;111;Magic number", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;111;assert width", SessionData.FP.FP);
        putDef(ret, "#bR;111;Width and height could be interchanged, test wont protect against it", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;111;Like above, more assertions are needed. Even though the method-under-test only is _intended_ to change split the height, this should still assert other dimensions, as well as contents.", SessionData.FP.FP);
        putDef(ret, "#bR;111;Same as above", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;115;I don't see where it tests for an empty sprite - it only seems to check the type, not content (line 125)?", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#bR;115;This test doesn't seem to test what it says", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#bR;118;I'm not familiar with the java coding style, but is it necessary to include it always?", SessionData.FP.FP);
        putDef(ret, "#bR;121;Change name to somethign that shows what test is doing instead of what you only want it to do.", SessionData.FP.FP);
        putDef(ret, "#bR;121;We should add another test for out of bounds that are smaller than 0", SessionData.FP.FP);
        putDef(ret, "#bR;121;again, test names should be better IMO", SessionData.FP.FP);
        putDef(ret, "#bR;122;this boilerplate code should be refactored into its own function or streamlined somehow: these 2 lines are repeated way too much", SessionData.FP.FP);
        putDef(ret, "#bR;123;The first 2 lines are the same in every test. Move to an @Before", P2.INIT_SPRITE);
        putDef(ret, "#bR;123;More magic string", SessionData.FP.FP);
        putDef(ret, "#bR;123;hardcoded path", SessionData.FP.FP);
        putDef(ret, "#bR;123;Tests should not rely on disk files or db.", SessionData.FP.FP);
        putDef(ret, "#bR;123;this should probably be a single method and not require two method calls: a constructor directly from a filename", P2.INIT_SPRITE);
        putDef(ret, "#bR;123;How are you guaranteeing this is out of bounds? You probably want ot mock the sprite.", P2.INIT_SPRITE);
        putDef(ret, "#bR;124;Magic Numbers", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;124;hardcoded numbers", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;124;This won't work; the test in the SUT is on x and y, but the invalid arguments you're passing in are width and height", SessionData.FP.FP);
        putDef(ret, "#bR;124;see above", SessionData.FP.FP);
        putDef(ret, "#bR;124;sprite should have a height of 64, and this will most likely cause the test to fail", SessionData.FP.FP);
        putDef(ret, "#bR;124;This line will go on the regular codepath, because (10,10) is within image. I’m not really sure what is the expected behaviour here, probably the split() function should verify that the entire split is within image, not only its top left pixel.", SessionData.FP.FP);
        putDef(ret, "#bR;124;Presumably either of the first two coordinates must be greater than or equal to 64.", SessionData.FP.FP);
        putDef(ret, "#bR;125;How does this assert that it's empty?", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;Won't work, it will still be a Sprite", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;We are only testing here if we get a sprite, not if it is empty", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;This verifies that you have a sprite - what verifies that it's empty?", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;redundant", P2.WRONG_ASSERT);
        putDef(ret, "#bR;126;Lots of duplicated lines of code on SpriteTest", P2.INIT_SPRITE);
        putDef(ret, "#bR;125;This doesn't test for an *empty* sprite.", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;At first look, this seems to always be true. Shouldn't the sprite's width and height be checked?", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;This assert is tautological; I think you mean \"instanceof EmptySprite\"", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;You are testing here if the \"new\" works, you should not test such obvious thing", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;this does not assert that the sprint is empty as the testdescription says", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;this doesn't test whether the sprite is empty: it tests that it's a sprite.", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;will always return true", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;This needs far more assertions to ensure the sprite is indeed empty. For example, asserting the width and height, assering that drawing the sprite draws nothing on a surface. The assert instanceof can be removed once assertions that test for properties of the Sprite object exist (these further tests will throw cast errors and nullpointerexceptions when given bad objects, so this assert will be unnessary).", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;This isn't checking to see if the sprit is of the correct dimensions, nor is there a test that specifies if it's okay for the sprite to be truncated", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;The same would succed for an in bounds case; this needs additional asserts.", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;should check if sprite is instance of EmptySprite", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;This does not seem to test for OutOfBounds, it should return an EmptySprite instance iirc", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;This line doesn’t seem to be testing that the sprite is empty", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;This doesn't actually assert that the new sprite is empty.", P2.WRONG_ASSERT);
        putDef(ret, "#bR;125;1. what is this even testing? 2. instanceof is heresy.", P2.WRONG_ASSERT);
        putDef(ret, "#aR;53;Subject of commit message not in present tense. Commit message unclear and missing all relevant information. Would not review such crap comits in real life.", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;68;Missing documentation.", P2.PROD_METHOD_DESCRIPTION);
        putDef(ret, "#aR;69;What if x or  y < 0 ?", P2.X_Y_GREATER_THAN_ZERO);
        putDef(ret, "#bR;110;s/13, 13/14,13/", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;125;Does not test waht doc says.", P2.WRONG_ASSERT);
        putDef(ret, "#bR;96;s/13, 13/13,14/", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#bR;98;Missing test for height.", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#aR;64;should return EmptySprite", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;79;Behavior changed; was EmptySprite before", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#bR;125;Always true; design is not useful; better throw exception or provide dedicated method", P2.WRONG_ASSERT);
        putDef(ret, "#aR;57;original returns empty sprite if outside range  - now it returns  an image sprite", P2.WRONG_RETURN_TYPE);
        putDef(ret, "#aR;57;width and height of the image is not used for this check, possible problems down the road?", P2.DIFFERENT_NUMBERS);
        putDef(ret, "#aR;64;multiple returns, introduce a single return statement", SessionData.FP.FP);
        putDef(ret, "#bR;101;a split sprite", P2.WRONG_ASSERT);
        return ret;
    }
}