require(foreign);require(lmtest);require(sandwich);require(oddsratio)
require(dplyr);require(effsize);require(car);require(pscl);require(pastecs)
require(MASS)

results <- read.table(
  file = "extracted-data/resultAnalysis-lw.csv", 
  sep = ',', 
  header = T,
  na.strings = c("NA", "-1"))


cresults = results %>%
            mutate(timeTotal = netTimeForReviewWithIdling/1000/60,
                   timeLogTime = (netTimeForReviewWithIdling)*log(netTimeForReviewWithIdling,base = exp(1)),
                   
                   TESTBugPropor = testBugsFound/testBugsTotal,
                   PRODBugPropor = prodBugsFound/prodBugsTotal,
                   
                   TESTIssuePropor = testMaintIssuesFound/testMaintIssuesTotal,
                   TESTIssueManualPropor = score_t_issues/2,
                   
                   TOTALbugsPropor = (prodBugsFound + testBugsFound) / (testBugsTotal + prodBugsTotal),
                   PRODIssuePropor = prodMaintIssuesFound/prodMaintIssuesTotal
                   ) %>%
              # remove outliers
              filter(timeTotal >= 1 & timeTotal <= 30 | is.na(timeTotal)) %>%
              # remove not so good reviewer
              filter(reviewer_id != '3') 

print_fit <- function(dataset,formula) {
  fitglm <- glm(formula=formula, family=binomial(logit), data=dataset)
  
  cov.m1 <- vcovHC(fitglm, type = "HC0")
  std.err <- sqrt(diag(cov.m1))
  q.val <- qnorm(0.975)
  r.est <- cbind(
    Estimate = coef(fitglm)
    , "Robust SE" = std.err
    , z = (coef(fitglm)/std.err)
    , "Pr(>|z|) "= 2 * pnorm(abs(coef(fitglm)/std.err), lower.tail = FALSE)
    , LL = coef(fitglm) - q.val  * std.err
    , UL = coef(fitglm) + q.val  * std.err
  )
  print(summary(fitglm))
  print("--------------- VIF, values should be < 10 to be OK")
  print(vif(fitglm))
  print("---------------")
  print(r.est)
  print("---------------")
  print(or_glm(data = dataset, model = fitglm, CI = 0.95))
  return(fitglm)
}

# descriptive statistics
distinct_res = cresults %>% distinct(reviewer_id,workStartHours,role,profDevExp,javaExp,reviewPract,programPract)

table(distinct_res$role)
table(distinct_res$profDevExp)
table(distinct_res$javaExp)
table(distinct_res$reviewPract)
table(distinct_res$programPract)

table(cresults$change,cresults$treatment)

print("------------------------------------MODEL-------------------------------------")

# Regression model for the proportion of bugs found in Test Code (Table 6, left side)
print_fit(cresults, cresults$TESTBugPropor ~ cresults$timeTotal + cresults$ifFirstReview + cresults$change + cresults$treatment + cresults$reviewPract + cresults$programPract + cresults$profDevExp + cresults$javaExp + cresults$workStartHours)

# Regression model for the proportion of bugs found in Production Code (Table not in the picture since nothing is statistically significant)
print_fit(cresults, cresults$PRODBugPropor ~ cresults$timeTotal + cresults$ifFirstReview + cresults$change + cresults$treatment + cresults$reviewPract + cresults$programPract + cresults$profDevExp + cresults$javaExp + cresults$workStartHours)

# Regression model for the proportion of maintainability issues found in Test Code (Table not in the picture since nothing is statistically significant)
print_fit(cresults, cresults$TESTIssuePropor ~ cresults$timeTotal + cresults$ifFirstReview + cresults$change + cresults$treatment + cresults$reviewPract + cresults$programPract + cresults$profDevExp + cresults$javaExp + cresults$workStartHours)

# Regression model for the proportion of maintainability issues found in Production Code (Table 6, right side)
print_fit(cresults, cresults$PRODIssuePropor ~ cresults$timeTotal + cresults$ifFirstReview + cresults$change + cresults$treatment + cresults$reviewPract + cresults$programPract + cresults$profDevExp + cresults$javaExp + cresults$workStartHours)


print("---------------------------------DESCRIPTIVES---------------------------------")

tf_results = cresults %>%
  filter(treatment == 'TF')

op_results = cresults %>%
  filter(treatment == 'OP')

pf_results = cresults %>%
  filter(treatment == 'PF')

# "-----------PRODBUG-----------"
# "-----------OVERALL-----------"
summary(cresults$PRODBugPropor)
# "-----------TF-----------"
summary(tf_results$PRODBugPropor)
# "-----------PF-----------"
summary(pf_results$PRODBugPropor)
# "-----------OP-----------"
summary(op_results$PRODBugPropor)

# TF vs. PF
wilcox.test(tf_results$PRODBugPropor,pf_results$PRODBugPropor)
# TF vs. OP
wilcox.test(tf_results$PRODBugPropor,op_results$PRODBugPropor)
# PF vs. OP
wilcox.test(pf_results$PRODBugPropor,op_results$PRODBugPropor)



# "-----------PRODISS-----------"
# "-----------OVERALL-----------"
summary(cresults$PRODIssuePropor)
# "-----------TF-----------"
summary(tf_results$PRODIssuePropor)
# "-----------PF-----------"
summary(pf_results$PRODIssuePropor)
# "-----------OP-----------"
summary(op_results$PRODIssuePropor)

# TF vs. PF
wilcox.test(tf_results$PRODIssuePropor,pf_results$PRODIssuePropor)
cliff.delta(tf_results$PRODIssuePropor,pf_results$PRODIssuePropor)
# TF vs. OP
wilcox.test(tf_results$PRODIssuePropor,op_results$PRODIssuePropor)
cliff.delta(tf_results$PRODIssuePropor,op_results$PRODIssuePropor)
# PF vs. OP
wilcox.test(pf_results$PRODIssuePropor,op_results$PRODIssuePropor)



# "-----------TESTBUG-----------"
# "-----------OVERALL-----------"
summary(cresults$TESTBugPropor)
# "-----------TF-----------"
summary(tf_results$TESTBugPropor)
# "-----------PF-----------"
summary(pf_results$TESTBugPropor)

# TF vs. PF
wilcox.test(tf_results$TESTBugPropor,pf_results$TESTBugPropor)
cliff.delta(tf_results$TESTBugPropor,pf_results$TESTBugPropor)

# "-----------TESTISS-----------"
# "-----------OVERALL-----------"
summary(cresults$TESTIssuePropor)
# "-----------TF-----------"
summary(tf_results$TESTIssuePropor)
# "-----------PF-----------"
summary(pf_results$TESTIssuePropor)

# TF vs. PF
wilcox.test(tf_results$TESTIssuePropor,pf_results$TESTIssuePropor)



# "-----------TIME--------------"
# "-----------OVERALL-----------"
summary(cresults$timeTotal)
# "-----------TF-----------"
summary(tf_results$timeTotal)
# "-----------PF-----------"
summary(pf_results$timeTotal)
# "-----------OP-----------"
summary(op_results$timeTotal)

# TF vs. PF
wilcox.test(tf_results$timeTotal,pf_results$timeTotal)

