package tdr.project.game;

import tdr.project.level.Level;
import tdr.project.level.PlayerFactory;
import tdr.project.level.Level;
import tdr.project.level.PlayerFactory;

/**
 * Factory that provides Game objects.
 *
 *
 */
public class GameFactory {

    /**
     * The factory providing the player objects.
     */
    private final PlayerFactory playerFactory;

    /**
     * Creates a new game factory.
     *
     * @param playerFactory
     *            The factory providing the player objects.
     */
    public GameFactory(PlayerFactory playerFactory) {
        this.playerFactory = playerFactory;
    }

    /**
     * Creates a game for a single level with one player.
     *
     * @param level
     *            The level to create a game for.
     * @return A new single player game.
     */
    public Game createSinglePlayerGame(Level level) {
        return new SinglePlayerGame(playerFactory.createPacMan(), level);
    }

    /**
     * Returns the player factory associated with this game factory.
     *
     * @return The player factory associated with this game factory.
     */
    protected PlayerFactory getPlayerFactory() {
        return playerFactory;
    }
}
